﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;

public partial class _Default : System.Web.UI.Page 
{
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!Page.IsPostBack)
        {
            // If the IP or hostname of the house has been passed in 
            // The backup is if the page is running locally within the house
            TextBoxIpHouse.Text = Request.QueryString["destination"] != null ? Request.QueryString["destination"].ToString().Trim() : "192.168.1.100";

            // If this was a call from LogEntries, run the alert sequence
            // The call to this page would have looked like: page.aspx?logentries=alert
            if (Request.QueryString["logentries"] != null)
            {
                string logEntriesValue = Request.QueryString["logentries"].ToString().Trim();
                if (logEntriesValue == "alert")
                {
                    // Flash the light red (and then off) 3 times to indicate the alarm
                    for (int i = 0; i < 3; i++)
                    {
                        LightsAction("turnon", 0);
                        LightsAction("setcolour", 175);
                        System.Threading.Thread.Sleep(1000);
                        LightsAction("turnoff", 0);
                        System.Threading.Thread.Sleep(1000);
                    }
                    // Set the light back to a calming blue, while we go address the alarm
                    LightsAction("turnon", 0);
                    LightsAction("setcolour", 10);
                }
            }


            // If a call has been made via querystring for immediate action
            if (Request.QueryString["action"] != null)
            {
                string actionRequired = Request.QueryString["action"].ToString().Trim();
                int colourRequired = Request.QueryString["colour"] != null ? Convert.ToInt32(Request.QueryString["colour"].ToString()) : 0;
                LightsAction(actionRequired, colourRequired);
            }

            // Keep a local copy of all params which were passed
            string incoming = "";
            foreach (string key in Request.Form.Keys) { incoming += key + ":" + Request.Form[key].ToString() + "<br>"; }
            foreach (string key in Request.QueryString.Keys) { incoming += key + ":" + Request.QueryString[key].ToString() + "<br>"; }
            if (incoming.Length > 0)
            {
                LiteralIncoming.Text = incoming;
                File.AppendAllText(MapPath("~/incomingparams.txt"), incoming);
            }

        }
    }

    /// <summary>
    /// Perform a standard action for the lights
    /// The UDP datagram sent is 3 bytes long and uses codes as per the manufacturer
    /// </summary>
    /// <param name="actionRequired"></param>
    /// <param name="colourRequired">An int from 0 to 255. These are special codes as per the manufaturer. Eg. 10=Blue, 175=Red</param>
    protected void LightsAction(string actionRequired, int colourRequired)
    {
        string destination = TextBoxIpHouse.Text.Trim();
        int destinationPort = 50000;
        if (actionRequired == "turnon") {SendSignalUdp(new byte[] {0x22, 0x0, 0x55}, destination, destinationPort);}
        if (actionRequired == "turnoff") { SendSignalUdp(new byte[] { 0x21, 0x0, 0x55 }, destination, destinationPort); }
        if (actionRequired == "setcolour") { SendSignalUdp(new byte[] { 0x20, byte.Parse(colourRequired.ToString()), 0x55 }, destination, destinationPort); }
        // Insert a pause before sending the next signals as they can interfere if rushed
        System.Threading.Thread.Sleep(100);
    }

    /// <summary>
    /// Send the UDP datagram
    /// </summary>
    /// <param name="byteArray"></param>
    /// <param name="destination"></param>
    /// <param name="destinationPort"></param>
    protected void SendSignalUdp(byte[] byteArray, string destination, int destinationPort)
    {
        System.Net.Sockets.UdpClient udpClient = new System.Net.Sockets.UdpClient(destination, destinationPort);
        udpClient.Send(byteArray, byteArray.Length);
        udpClient.Close();
    }



    // Interective button controls

    protected void Button1_Click(object sender, EventArgs e)
    {
        // Light on
        LightsAction("turnon", 0);
        LightsAction("setcolour", Convert.ToInt32(TextBoxColour.Text));
    }

    protected void Button2_Click(object sender, EventArgs e)
    {
        // Light off
        LightsAction("turnoff", 0);
    }

    protected void Button3_Click(object sender, EventArgs e)
    {
        LightsAction("turnon", 0);
        // Cycle the light through colours
        for (int i = 0; i < 256; i = i + 40)
        {
            System.Threading.Thread.Sleep(1000);
            LightsAction("setcolour", i);

        }
        LightsAction("turnoff", 0);
    }
}